@@ -1,5 +1,7 @@
 /* linux.c -- Linux specific code for ifconfig
-  Copyright (C) 2001-2020 Free Software Foundation, Inc.
+  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
+  2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021
+  Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -41,7 +43,9 @@
 #include <sys/ioctl.h>
 #include <net/if.h>
 #include <net/if_arp.h>
-#include <linux/if_ether.h>
+#ifdef HAVE_NETINET_ETHER_H
+# include <netinet/ether.h>
+#endif
 
 #include <read-file.h>
 #include <unused-parameter.h>
@@ -927,7 +931,8 @@ linux_if_nameindex (void)
   if (fd < 0)
     return NULL;
 
-  content = read_file (PATH_PROCNET_DEV, &length);
+  /* Read a public text file.  */
+  content = read_file (PATH_PROCNET_DEV, 0, &length);
   if (content == NULL)
     return NULL;
 
