@@ -1,5 +1,7 @@
 /*
-  Copyright (C) 2000-2020 Free Software Foundation, Inc.
+  Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+  2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
+  2020, 2021 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -64,7 +66,6 @@
 #include <grp.h>
 #include <termios.h>
 
-#include <intprops.h>
 #include <inttostr.h>
 #include "ls.h"
 #include "extern.h"
@@ -119,7 +120,6 @@ ls_main (int argc, char **argv)
   static char dot[] = ".", *dotav[] = { dot, NULL };
   struct winsize win;
   int ch, fts_options;
-  int kflag = 0;
   char *p;
 
   /*
@@ -227,7 +227,6 @@ ls_main (int argc, char **argv)
 	  break;
 	case 'k':
 	  blocksize = 1024;
-	  kflag = 1;
 	  break;
 	case 'o':
 	  f_flags = 1;
