@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2009-2020 Free Software Foundation, Inc.
+  Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017,
+  2018, 2019, 2020, 2021 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -114,9 +115,9 @@ parse_level (char *str)
   if (p)
     *p++ = 0;
 
-  fac = decode (str, facilitynames, "facility");
+  fac = decode (str, (CODE *) facilitynames, "facility");
   if (p)
-    prio = decode (p, prioritynames, "priority");
+    prio = decode (p, (CODE *) prioritynames, "priority");
 
   return MAKE_PRI (fac, prio);
 }
