@@ -1,5 +1,8 @@
 /*
-  Copyright (C) 1993-2020 Free Software Foundation, Inc.
+  Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+  2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012,
+  2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Free Software
+  Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -634,6 +637,7 @@ willoption (int option)
 	      slctab[SLC_XON].defset.flag |= SLC_DEFAULT;
 	      slctab[SLC_XOFF].defset.flag &= ~SLC_LEVELBITS;
 	      slctab[SLC_XOFF].defset.flag |= SLC_DEFAULT;
+	      /* FALLTHROUGH */
 	    case TELOPT_TTYPE:
 	    case TELOPT_SGA:
 	    case TELOPT_NAWS:
