@@ -1,5 +1,6 @@
 /*
-  Copyright (C) 2009-2020 Free Software Foundation, Inc.
+  Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017,
+  2018, 2019, 2020, 2021 Free Software Foundation, Inc.
 
   This file is part of GNU Inetutils.
 
@@ -80,6 +81,11 @@ const char *hide_strings[] = {
   "Signature Domains' Whois Service", "agree to abide by the above",
   "Access to ASNIC", "by this policy.",
   "* Copyright (C) 1998 by SGNIC", "* modification.",
+  "% The RIPE Database is subject", "\n",
+  "% This query was served", "\n",
+  "% Copyright LACNIC", "%  only for lawful",
+  "% Copyright (c) Nic.br", "%  any similar purpose",
+  "# ARIN WHOIS data and services", "# Copyright 19",
   NULL, NULL
 };
 
@@ -107,6 +113,11 @@ struct ip_del ip_assign[] = {
   {0, 0, NULL}
 };
 
+struct ip_del ip6_assign[] = {
+#include "ip6_del.h"
+  {0, 0, NULL}
+};
+
 struct as_del
 {
   unsigned short first;
